local Emphasize = Grid:GetModule("GridStatus"):GetModule("GridStatusEmphasize")
local GridRoster = Grid:GetModule("GridRoster")
local GridFrame = Grid:GetModule("GridFrame")

local GridStatusEmphasize_Auras = Emphasize:NewModule("GridStatusEmphasize_Auras", "AceEvent-3.0")

-- upvalues
local UnitDebuff = UnitDebuff
local UnitBuff = UnitBuff
local UnitGUID = UnitGUID

-- data
local settings


GridStatusEmphasize_Auras.defaultDB = {
	range = true,
	auras = { },
}

local options = {
	["range"] = {
		order = 1,
		type = "toggle",
		name = "Range filter",
		desc = "Don't emphasize units out of range, even when they are below the threshold",
		get = function () return GridStatusEmphasize_Auras.db.profile.range end,
		set = function (_, v) GridStatusEmphasize_Auras.db.profile.range = v end,
	},
	["add"] = {
		order = 2,
		type = "input",
		width = "double",
		name = "Add new aura",
		set = function(_, v) GridStatusEmphasize_Auras:AddAura(v) end,
	},
	["auras"] = {
		order = 3,
		type = "group",
		name = "Auras",
		inline = true,
		args = { },
	}
}

function GridStatusEmphasize_Auras:AddAura(name)
	table.insert(settings.auras, name)
	
	self:UpdateOptions()
end

function GridStatusEmphasize_Auras:RemoveAura(index)
	table.remove(settings.auras, index)
	
	self:UpdateOptions()
end

function GridStatusEmphasize_Auras:UpdateOptions()
	options.auras.args = { }
	
	for i, aura in ipairs(settings.auras) do
		options.auras.args[aura] = {
			order = i,
			type = "execute",
			width = "double",
			name = aura,
			desc = "Remove " .. aura,
			func = function() return self:RemoveAura(i) end,
		}
	end
end

function GridStatusEmphasize_Auras:OnInitialize()
	self.super.OnInitialize(self)
	
	self:RegisterOptions("Auras", options)
	settings = self.db.profile
	
	self:RegisterEvent("UNIT_AURA", "ScanUnit")
	self:RegisterEvent("Grid_UnitJoined")
	
	self:UpdateOptions()
end

function GridStatusEmphasize_Auras:Grid_UnitJoined(event, guid, unitid)
	self:UpdateUnit("Grid_UnitJoined", unitid)
end

function GridStatusEmphasize_Auras:UpdateAll()
	Emphasize:Pause()
	
	for _, aura in ipairs(settings.auras) do
		Emphasize:TempDeemphasizeAllUnits("aura " .. aura)
	end
	
	for guid, unitid in GridRoster:IterateRoster() do
		self:ScanUnit("UpdateAllUnits", unitid, guid)
	end
	
	Emphasize:Resume()
end

function GridStatusEmphasize_Auras:ScanUnit(event, unitid, unitguid)
	unitguid = unitguid or UnitGUID(unitid)
	if not GridRoster:IsGUIDInRaid(unitguid) then
		return
	end
	
	for _, aura in ipairs(settings.auras) do
		local name = UnitDebuff(unitid, aura)
		
		if not name then
			name = UnitBuff(unitid, aura)
		end
		
		if name and (not settings.range or GridFrame:UnitInRange(unitid, 40)) then
			Emphasize:TempEmphasizeUnit(unitguid, "aura " .. aura)
		else
			Emphasize:TempDeemphasizeUnit(unitguid, "aura " .. aura)
		end
	end
end
